@@ -17,7 +17,12 @@
 #endif
 
 #include <openacc.h>
-void atlas_acc_map_data(void* cpu_ptr, void* gpu_ptr, unsigned long size)
-{
+
+void atlas_acc_map_data(void* cpu_ptr, void* gpu_ptr, unsigned long size) {
   acc_map_data(cpu_ptr, gpu_ptr, size);
 }
+
+
+void atlas_acc_unmap_data(void* cpu_ptr) {
+  acc_unmap_data(cpu_ptr);
+}
